@@ -11,7 +11,7 @@ use warnings;
 
 our @ISA = qw(Exporter);
 our @EXPORT = qw(bg_eval bg_qx);
-our $VERSION = '0.53';
+our $VERSION = '0.54';
 
 sub _choose_protocol {
   if (CONFIG_module('YAML')) {
@@ -30,46 +30,47 @@ sub _choose_protocol {
 }
 
 sub bg_eval (&;@) {
-  my $proto = _choose_protocol();
-  if (!defined $proto) {
-    croak "Forks::Super: bg_eval call requires either YAML or JSON\n";
-  }
-  my ($code, @other_options) = @_;
-  my %other_options;
-  if (@other_options > 0 && ref $other_options[0] eq 'HASH') {
-      %other_options = %{$other_options[0]};
-  } else {
-      %other_options = @other_options;
-  }
-  if (defined($other_options{daemon}) && $other_options{daemon}) {
-      croak "Forks::Super::bg_eval: daemon option not allowed on bg_eval call";
-  }
-
-  if ($Forks::Super::SysInfo::SLEEP_ALARM_COMPATIBLE <= 0) {
-    # timeout, expiration are incompatible with bg_eval
-    foreach (keys %other_options) {
-      if ($_ eq "timeout" || $_ eq "expiration") {
-	croak "Forks::Super::bg_eval: ",
-	  "$_ option not allowed because ",
-	  "alarm/sleep are not compatible on this system.\n";
-      }
+    my ($code, @other_options) = @_;
+    my $proto = _choose_protocol();
+    if (!defined $proto) {
+	croak "Forks::Super: bg_eval call requires either YAML or JSON\n";
+    }
+    my %other_options;
+    if (@other_options > 0 && ref $other_options[0] eq 'HASH') {
+	%other_options = %{$other_options[0]};
+    } else {
+	%other_options = @other_options;
+    }
+    if (defined($other_options{daemon}) && $other_options{daemon}) {
+	croak 'Forks::Super::bg_eval: ',
+	    'daemon option not allowed on bg_eval call';
     }
-  }
 
-  my $p = $$;
-  my ($result, @result);
+    if ($Forks::Super::SysInfo::SLEEP_ALARM_COMPATIBLE <= 0) {
+	# timeout, expiration are incompatible with bg_eval
+	foreach (keys %other_options) {
+	    if ($_ eq 'timeout' || $_ eq 'expiration') {
+		croak 'Forks::Super::bg_eval: ',
+		"$_ option not allowed because ",
+		"alarm/sleep are not compatible on this system.\n";
+	    }
+	}
+    }
 
-  require Forks::Super::Tie::BackgroundScalar;
-  $result = Forks::Super::Tie::BackgroundScalar->new(
-      'eval', $code, 
-      protocol => $proto,
-      %other_options);
-  if ($$ != $p) {
-      # a WTF observed on Windows
-      croak "Forks::Super::bg_eval: ",
-	"Inconsistency in process IDs: $p changed to $$!\n";
-  }
-  return $result;
+    my $p = $$;
+    my ($result, @result);
+
+    require Forks::Super::Tie::BackgroundScalar;
+    $result = Forks::Super::Tie::BackgroundScalar->new(
+	'eval', $code, 
+	protocol => $proto,
+	%other_options);
+    if ($$ != $p) {
+	# a WTF observed on Windows
+	croak 'Forks::Super::bg_eval: ',
+	    "Inconsistency in process IDs: $p changed to $$!\n";
+    }
+    return $result;
 }
 
 sub bg_qx {
@@ -82,13 +83,13 @@ sub bg_qx {
   }
 
   if (defined($other_options{daemon}) && $other_options{daemon}) {
-      croak "Forks::Super::bg_qx: daemon option not allowed on bg_qx call";
+      croak 'Forks::Super::bg_qx: daemon option not allowed on bg_qx call';
   }
   if ($Forks::Super::SysInfo::SLEEP_ALARM_COMPATIBLE <= 0) {
     # timeout, expiration are incompatible with bg_qx
     foreach (keys %other_options) {
-      if ($_ eq "timeout" || $_ eq "expiration") {
-	croak "Forks::Super::bg_qx: ",
+      if ($_ eq 'timeout' || $_ eq 'expiration') {
+	croak 'Forks::Super::bg_qx: ',
 	  "$_ option not allowed because ",
 	  "alarm/sleep are not compatible on this system.\n";
       }
@@ -103,7 +104,7 @@ sub bg_qx {
       'qx', $command, %other_options);
   if ($$ != $p) {
       # a WTF observed on Windows
-      croak "Forks::Super::bg_qx: ",
+      croak 'Forks::Super::bg_qx: ',
 	"Inconsistency in process IDs: $p changed to $$!\n";
   }
   return $result;
