@@ -29,11 +29,14 @@ use strict;
 use warnings;
 
 our @ISA = qw(Exporter IO::Handle);
+our $VERSION = '0.54';
 
 sub TIEHANDLE {
   my ($class, %props) = @_;
   my $self = bless Forks::Super::Job::Ipc::_gensym(), $class;
-  $$self->{$_} = $props{$_} for keys %props;
+  foreach my $attr (keys %props) {
+      $$self->{$attr} = $props{$attr};
+  }
   $$self->{created} = Time::HiRes::time();
   return $self;
 }
@@ -45,7 +48,7 @@ sub OPEN {
   $$self->{OPEN}++;
   my ($result, $new_err);
   my $old_err = $!;
-  if ($$self->{OPENED}) {
+  if ($$self->{is_opened}) {
     # filehandle already open, implicit close
     $self->CLOSE;
   }
@@ -63,8 +66,13 @@ sub OPEN {
     } else {
       $result = open *$self, $mode;         ## no critic (TwoArgOpen)
     }
-    $$self->{opened} = ($$self->{OPENED} = $result) && Time::HiRes::time();
-    $$self->{closed} = 1 if !$result;
+    $$self->{opened} = ($$self->{is_opened} = $result) && Time::HiRes::time();
+    if (!$result) {
+	$$self->{closed} = "open($mode,$expr) failed: $!";
+    } else {
+	# $$self->{closed} could be defined from an earlier, failed open attempt
+	delete $$self->{closed};
+    }
     $$self->{open_error} = $new_err = $!;
   }
   $! = $new_err || $old_err;
@@ -72,101 +80,103 @@ sub OPEN {
 }
 
 sub BINMODE {
-  my ($self, $layer) = @_;
-  $$self->{BINMODE}++;
-  return binmode *$self, $layer || ":raw";
+    my ($self, $layer) = @_;
+    $$self->{BINMODE}++;
+    return binmode *$self, $layer || ':raw';
 }
 
 sub READLINE {
-  my $self = shift;
-  $$self->{READLINE}++;
-  return <$self>;
+    my $self = shift;
+    $$self->{READLINE}++;
+    return <$self>;
 }
 
 sub FILENO {
-  my $self = shift;
-  $$self->{FILENO}++;
-  return $$self->{fileno} ||= CORE::fileno($self);
+    my $self = shift;
+    $$self->{FILENO}++;
+    return $$self->{fileno} ||= CORE::fileno($self);
 }
 
 sub SEEK {
-  my ($self, $whence, $position) = @_;
-  $$self->{SEEK}++;
-  return seek $self, $whence, $position;
+    my ($self, $whence, $position) = @_;
+    $$self->{SEEK}++;
+    return seek $self, $whence, $position;
 }
 
 sub GETC {
-  my $self = shift;
-  $$self->{GETC}++;
-  return getc($self);
+    my $self = shift;
+    $$self->{GETC}++;
+    return getc($self);
 }
 
 sub READ {
-  my ($self, undef, $length, $offset) = @_;
-  $$self->{READ}++;
-  return read $self, $_[1], $length, $offset;
+    my ($self, undef, $length, $offset) = @_;
+    $$self->{READ}++;
+    return read $self, $_[1], $length, $offset;
 }
 
 sub PRINT {
-  my $self = shift;
-  $$self->{PRINT}++;
-  if ($$self->{closed}) {
+    my ($self,@msg) = @_;
+    $$self->{PRINT}++;
+    if ($$self->{closed}) {
 
-    carp "print on closed fh ", *$self, " ", 
-      $$self->{name}||'', " closed=",$$self->{closed},"\n";
-    return;
+	carp 'print on closed fh ', *$self, ' ', 
+	    $$self->{name}||'', ' closed=',$$self->{closed},"\n";
+	return;
 
-  }
-  my $z = print {$self} @_;
-  IO::Handle::flush($self);
-  return $z;
+    }
+    my $z = print {$self} @msg;
+    IO::Handle::flush($self);
+    return $z;
 }
 
 sub PRINTF {
-  my $self = shift;
-  $$self->{PRINTF}++;
-  if ($$self->{closed}) {
+    my ($self,$template,@args) = @_;
+    $$self->{PRINTF}++;
+    if ($$self->{closed}) {
 
-    carp "printf on closed fh ", $$self->{name}, "\n";
-    return;
+	carp 'printf on closed fh ', $$self->{name}, "\n";
+	return;
 
-  }
-  seek $self, 0, 2;
-  return printf {$self} @_;
+    }
+    seek $self, 0, 2;
+    return printf {$self} $template, @args;
 }
 
 sub TELL {
-  my $self = shift;
-  $$self->{TELL}++;
-  return tell $self;
+    my $self = shift;
+    $$self->{TELL}++;
+    return tell $self;
 }
 
 sub WRITE {
-  my ($self, $string, $length, $offset) = @_;
-  $$self->{WRITE}++;
-  if ($$self->{closed}) {
+    my ($self, $string, $length, $offset) = @_;
+    $$self->{WRITE}++;
+    if ($$self->{closed}) {
 
-    carp "write/syswrite on closed fh ", $$self->{name}, "\n";
-    return;
+	carp 'write/syswrite on closed fh ', $$self->{name}, "\n";
+	return;
 
-  }
-  seek $self, 0, 2;
-  return syswrite $self, $string, $length||length($string), $offset||0;
+    }
+    seek $self, 0, 2;
+    return syswrite $self, $string, $length||length($string), $offset||0;
 }
 
 sub CLOSE {
-  my $self = shift;
-  $$self->{CLOSE}++;
-  if (!$$self->{closed}) {
-    $$self->{closed} = Time::HiRes::time();
+    my $self = shift;
+    $$self->{CLOSE}++;
 
-    my $elapsed = $$self->{closed} - $$self->{opened}||$$self->{created}||$^T;
-    $$self->{elapsed} = $elapsed;
-    my $result = close $self;
-    delete $$self->{OPENED};
-    return $result;
-  }
-  return;
+    if (!$$self->{closed}) {
+	$$self->{closed} = Time::HiRes::time();
+
+	my $elapsed = 
+	    $$self->{closed} - $$self->{opened}||$$self->{created}||$^T;
+	$$self->{elapsed} = $elapsed;
+	my $result = close $self;
+	delete $$self->{is_opened};
+	return $result;
+    }
+    return;
 }
 
 sub EOF {
@@ -184,31 +194,34 @@ sub UNTIE {
     my ($self, $existing_inner_references) = @_;
     if ($existing_inner_references > 1) {
 	if (!$Forks::Super::Job::INSIDE_END_QUEUE) {
-	    warn "untie attempted while ", $existing_inner_references,
-	    	" still exist";
+	    warn 'untie attempted while ', $existing_inner_references,
+	    	' still exist';
 	}
     }
+    return;
 }
 
 sub Forks::Super::Tie::Delegator::AUTOLOAD {
-  my $method = $Forks::Super::Tie::Delegator::AUTOLOAD;
-  $method =~ s/.*:://;
-  my $tied = tied *{shift @_};
-  if ($tied) {
-    return eval "\$tied->$method(\@_)" or do {};  ## no critic (StringyEval)
-  }
+    my $tied = shift;
+    $tied = tied *{$tied};
+    #my $tied = tied *{shift @_};
+    my $method = $Forks::Super::Tie::Delegator::AUTOLOAD;
+    $method =~ s/.*:://;
+    if ($tied) {
+	return eval "\$tied->$method(\@_)" or do {};  ## no critic (StringyEval)
+    }
 
-  if ($method eq 'DESTROY') {
-      return;  # no op
-  }
+    if ($method eq 'DESTROY') {
+	return;  # no op
+    }
 
-  if (!$Forks::Super::Job::INSIDE_END_QUEUE) {
-    Carp::confess "Can't delegate method $method from an untied object!";
-  }
+    if (!$Forks::Super::Job::INSIDE_END_QUEUE) {
+	Carp::confess "Can't delegate method $method from an untied object!";
+    }
 
-  Carp::cluck "Delegation of $method requested for untied object ",
-    	"during global destruction ...";
-  return;
+    Carp::cluck "Delegation of $method requested for untied object ",
+    	'during global destruction ...';
+    return;
 }
 
 1;
