@@ -5,7 +5,7 @@
 
 package Forks::Super::Wait;
 use Forks::Super::Job;
-use Forks::Super::Util qw(is_number isValidPid pause IS_WIN32);
+use Forks::Super::Util qw(is_number isValidPid IS_WIN32);
 use Forks::Super::Debug qw(:all);
 use Forks::Super::Config;
 use Forks::Super::Queue;
@@ -20,36 +20,13 @@ use warnings;
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(wait waitpid waitall TIMEOUT WREAP_BG_OK);
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
-our $VERSION = '0.53';
+our $VERSION = '0.54';
 
-our ($productive_pause_code, $productive_waitpid_code);
-
-
-# 5.8 compatible function equivalent to 5.10's  $x // $y  construction
-sub _defined_or {
-    my ($target, $default) = @_;
-    return defined($target) ? $target : $default;
-}
-
-# What should  waitpid  return for certain inputs, including bogus, inputs,
-# and what are the side-effects on $? ?
-my ($ACTIVE_WAITPID_RESULT, $ACTIVE_WAITPID_STATUS,
-    $REAPED_WAITPID_RESULT, $REAPED_WAITPID_STATUS,
-    $BOGUS_WAITPID_RESULT, $BOGUS_WAITPID_STATUS) =
-    (_defined_or($Forks::Super::SysInfo::ACTIVE_WAITPID_RESULT, 0),
-     _defined_or($Forks::Super::SysInfo::ACTIVE_WAITPID_STATUS, -1),
-     _defined_or($Forks::Super::SysInfo::REAPED_WAITPID_RESULT, -1),
-     _defined_or($Forks::Super::SysInfo::REAPED_WAITPID_STATUS, -1),
-     _defined_or($Forks::Super::SysInfo::BOGUS_WAITPID_RESULT, -1),
-     _defined_or($Forks::Super::SysInfo::BOGUS_WAITPID_STATUS, -1));
+my ($productive_waitpid_code);
 
 tie our $WAIT_ACTION_ON_SUSPENDED_JOBS, 
     'Forks::Super::Tie::Enum', qw(wait fail resume);
 
-sub set_productive_pause_code (&) {
-    $productive_pause_code = shift;
-    return;
-}
 sub set_productive_waitpid_code (&) {
     $productive_waitpid_code = shift;
     return;
@@ -58,35 +35,44 @@ sub set_productive_waitpid_code (&) {
 use constant TIMEOUT => -1.5;
 use constant ONLY_SUSPENDED_JOBS_LEFT => -1.75;
 use constant WREAP_BG_OK => WNOHANG() << 1;
+use constant _BRIEF_TIMEOUT => 1E-6; # seconds
+use constant FOREVER => 9.0E9;       # to a first order 285 years is forever
 
 sub wait {
     my $timeout = shift || 0;
-    $timeout = 1E-6 if $timeout < 0;
-
-    # print STDERR "invoked Forks::Super::wait  (\$DEBUG=$DEBUG)\n";
-
-    debug("invoked Forks::Super::wait") if $DEBUG;
+    if ($timeout < 0) {
+	$timeout = _BRIEF_TIMEOUT;
+    }
+    debug('invoked Forks::Super::wait') if $DEBUG;
     return Forks::Super::Wait::waitpid(-1, 0, $timeout);
 }
 
-sub waitpid {
-    my ($target,$flags,$timeout,@dummy) = @_;
-
-    $productive_waitpid_code->() if $productive_waitpid_code;
-    $timeout = 0 if !defined $timeout;
-    $timeout = 1E-6 if $timeout < 0;
-
-    if (@dummy > 0) {
+sub _cleanse_waitpid_args {
+    my ($flags, $timeout, @dummy) = @_;
+    $timeout ||= 0;
+    if ($timeout < 0) {
+	$timeout = _BRIEF_TIMEOUT;
+    }
+    if (@dummy > 2) {
 	carp "Forks::Super::waitpid: Too many arguments\n";
     }
     if (not defined $flags) {
 	carp "Forks::Super::waitpid: Not enough arguments\n";
 	$flags = 0;
     }
+    return ($flags, $timeout);
+}
+
+sub waitpid {
+    my $target = shift;
+    my ($flags, $timeout) = _cleanse_waitpid_args(@_);
+
+    __run_productive_waitpid_code();
 
     # waitpid:
     #   -1:    wait on any process
     #   t>0:   wait on process #t
+    #   name:  wait on any process with the name
     #    0:    wait on any process in current process group
     #   -t:    wait on any process in process group #t
 
@@ -95,47 +81,41 @@ sub waitpid {
     my $reap_bg_ok = $flags == WREAP_BG_OK;
 
     if (is_number($target) && $target == -1) {
-	debug("waitpid: dispatching _waitpid_any") if $DEBUG;
+	debug('waitpid: dispatching _waitpid_any') if $DEBUG;
 	return _waitpid_any($no_hang, $reap_bg_ok, $timeout);
     }
     if (defined $ALL_JOBS{$target}) {
-	debug("waitpid: dispatching _waitpid_target") if $DEBUG;
+	debug('waitpid: dispatching _waitpid_target') if $DEBUG;
 	return _waitpid_target($no_hang, $reap_bg_ok, $target, $timeout);
     }
     if (0 < (my @wantarray = Forks::Super::Job::getByName($target))) {
-	debug("waitpid: dispatching _waitpid_name") if $DEBUG;
+	debug('waitpid: dispatching _waitpid_name') if $DEBUG;
 	return _waitpid_name($no_hang, $reap_bg_ok, $target, $timeout);
     }
-    if (!is_number($target)) {
-	debug("waitpid: bogus target") if $DEBUG;
-	return _bogus_waitpid_result();
-    }
-    if ($target > 0) {
-	debug("waitpid: bogus target") if $DEBUG;
+    if (!is_number($target) || $target > 0) {
+	debug("waitpid: bogus target $target") if $DEBUG;
 	return _bogus_waitpid_result();
     }
+
     if ($Forks::Super::SysInfo::CONFIG{'getpgrp'}) {
-	if ($target == 0) {
-	    unless (eval { $target = getpgrp(0) } ) {
-		$target = -$$;
-	    }
-	} else {
-	    $target = -$target;
-	}
-	debug("waitpid: dispatching _waitpid_pgrp") if $DEBUG;
+	# $target is a number <= 0
+
+	debug('waitpid: dispatching _waitpid_pgrp') if $DEBUG;
 	return _waitpid_pgrp($no_hang, $reap_bg_ok, $target, $timeout);
-    } else {
-	debug("waitpid: bogus (pgid) target") if $DEBUG;
-	return _bogus_waitpid_result();
     }
+
+    debug('waitpid: bogus (pgid) target') if $DEBUG;
+    return _bogus_waitpid_result();
 }
 
 sub waitall {
-    my $timeout = shift || 9E9;  # 285 years should be long enough to wait
-    $timeout = 1E-6 if $timeout < 0;
+    my $timeout = shift || &FOREVER;
+    if ($timeout < 0) {
+	$timeout = &_BRIEF_TIMEOUT;
+    }
     my $waited_for = 0;
     my $expire = Time::HiRes::time() + $timeout ;
-    debug("waitall: waiting on all procs") if $DEBUG;
+    debug('waitall: waiting on all procs') if $DEBUG;
     my $pid;
     do {
 	# $productive_waitpid_code->() if $productive_waitpid_code;
@@ -181,7 +161,7 @@ sub _reap_return {
 #
 sub _reap {
     my ($reap_bg_ok, $optional_pgid) = @_; # to reap procs from specific group
-    $productive_waitpid_code->() if $productive_waitpid_code;
+    __run_productive_waitpid_code();
     Forks::Super::Sigchld::handle_bastards();
 
     my @j = @ALL_JOBS;
@@ -239,7 +219,7 @@ sub _reap {
 # wait on any process
 sub _waitpid_any {
     my ($no_hang,$reap_bg_ok,$timeout) = @_;
-    my $expire = Time::HiRes::time() + ($timeout || 9E9);
+    my $expire = Time::HiRes::time() + ($timeout || &FOREVER);
     my ($pid, $nactive2, $nalive, $nactive, $nalive2) = _reap($reap_bg_ok);
     if ($no_hang == 0) {
 	while (!isValidPid($pid,1) && $nalive > 0) {
@@ -253,23 +233,28 @@ sub _waitpid_any {
 		    # XXX - reset $? ?
 		    return ONLY_SUSPENDED_JOBS_LEFT;
 		} elsif ($WAIT_ACTION_ON_SUSPENDED_JOBS eq 'resume') {
-		    _active_one_suspended_job($reap_bg_ok);
+		    _activate_one_suspended_job($reap_bg_ok);
 		}
 	    }
-	    $productive_waitpid_code->() if $productive_waitpid_code;
-	    pause();
+	    __run_productive_waitpid_code();
+
+	    # XXX - $DEFAULT_PAUSE here? 
+	    # Pause time should not be greater than the timeout.
+	    Forks::Super::Util::pause();
 	    ($pid, $nactive2, $nalive, $nactive, $nalive2) = _reap($reap_bg_ok);
 	}
     }
     if (defined $ALL_JOBS{$pid}) {
 	my $job = Forks::Super::Job::get($ALL_JOBS{$pid});
-	pause() while not defined $job->{status};
+	while (not defined $job->{status}) {
+	    Forks::Super::Util::pause();
+	}
 	$? = $job->{status};
     }
     return $pid;
 }
 
-sub _active_one_suspended_job {
+sub _activate_one_suspended_job {
     my @suspended = grep { $_->{state} eq 'SUSPENDED' } @Forks::Super::ALL_JOBS;
     if (@suspended == 0) {
 	@suspended = grep { 
@@ -279,7 +264,7 @@ sub _active_one_suspended_job {
     @suspended = sort { 
 	$b->{queue_priority} <=> $a->{queue_priority} } @suspended;
     if (@suspended == 0) {
-	warn "Forks::Super::_activate_one_suspended_job(): ",
+	warn 'Forks::Super::_activate_one_suspended_job(): ',
 	" can't find an appropriate suspended job to resume\n";
 	return;
     }
@@ -291,81 +276,92 @@ sub _active_one_suspended_job {
 }
 
 sub _bogus_waitpid_result {
-    if (defined $BOGUS_WAITPID_STATUS) {
-	$? = $BOGUS_WAITPID_STATUS;
+    if (defined $Forks::Super::SysInfo::BOGUS_WAITPID_STATUS) {
+	$? = $Forks::Super::SysInfo::BOGUS_WAITPID_STATUS;
     }
-    return $BOGUS_WAITPID_RESULT;
+    return $Forks::Super::SysInfo::BOGUS_WAITPID_RESULT;
 }
 
 sub _active_waitpid_result {
-    if (defined $ACTIVE_WAITPID_STATUS) {
-	$? = $ACTIVE_WAITPID_STATUS;
+    if (defined $Forks::Super::SysInfo::ACTIVE_WAITPID_STATUS) {
+	$? = $Forks::Super::SysInfo::ACTIVE_WAITPID_STATUS;
     }
-    return $ACTIVE_WAITPID_RESULT;
+    return $Forks::Super::SysInfo::ACTIVE_WAITPID_RESULT;
 }
 
 sub _reaped_waitpid_result {
-    if (defined $REAPED_WAITPID_STATUS) {
-	$? = $REAPED_WAITPID_STATUS;
+    if (defined $Forks::Super::SysInfo::REAPED_WAITPID_STATUS) {
+	$? = $Forks::Super::SysInfo::REAPED_WAITPID_STATUS;
     }
-    return $REAPED_WAITPID_RESULT;
+    return $Forks::Super::SysInfo::REAPED_WAITPID_RESULT;
 }
 
 # wait on a specific process
 sub _waitpid_target {
     my ($no_hang, $reap_bg_ok, $target, $timeout) = @_;
-    my $expire = Time::HiRes::time() + ($timeout || 9E9);
-
-    #Forks::Super::handle_CHLD(-1);
+    my $expire = Time::HiRes::time() + ($timeout || &FOREVER);
 
     my $job = $ALL_JOBS{$target};
+
     if (not defined $job) {
-	debug("_waitpid_target: bogus target") if $job->{debug} & 2;
+	debug('_waitpid_target: bogus target') if $job->{debug} & 2;
 	return _bogus_waitpid_result();
-    }
+    } 
+
     if ($job->{state} eq 'COMPLETE') {
 	debug("_waitpid_target: job $job is complete, reaping ...")
 	    if $job->{debug} & 2;
 	$job->_mark_reaped;
 	return _reap_return($job);
-    } elsif ($job->{daemon}) {
+    }
+
+    if ($job->{daemon}) {
 	debug("_waitpid_target: job $job is a daemon, returning bogus result")
 	    if $job->{debug} & 2;
 	return _bogus_waitpid_result();
-    } elsif ($job->{state} eq 'REAPED') {
+    }
+
+    if ($job->{state} eq 'REAPED') {
 	debug("_waitpid_target: job $job is already reaped ...") 
 	    if $job->{debug} & 2;
 	return _reaped_waitpid_result();
-    } elsif ($no_hang) {
+    }
+
+    if ($no_hang) {
 	debug("_waitpid_target: job $job is still $job->{state}")
 	    if $job->{debug} & 2;
 	return _active_waitpid_result();
-    } else {
-	# block until job is complete.
-	debug("_waitpid_target: blocking until $job is complete")
-	    if $job->{debug} & 2;
-	while ($job->{state} ne 'COMPLETE' and $job->{state} ne 'REAPED') {
-	    if (Time::HiRes::time() >= $expire) {
-		# XXX - update $? ?
-		return TIMEOUT;
-	    }
-	    $productive_waitpid_code->() if $productive_waitpid_code;
-	    pause();
-	    #Forks::Super::Sigchld::handle_CHLD(-1) if &IS_WIN32;
-	    if ($job->{state} =~ /DEFER|SUSPEND/) {
-		Forks::Super::Queue::check_queue();
-	    }
-	}
-	debug("_waitpid_target: job $job is complete now, reaping ...")
-	    if $job->{debug} & 2;
-	$job->_mark_reaped;
-	return _reap_return($job);
     }
+
+    # block until job is complete.
+    debug("_waitpid_target: blocking until $job is complete")
+	    if $job->{debug} & 2;
+
+    my $block = _block_until_job_completes($job, $expire);
+    return TIMEOUT if $block == TIMEOUT;
+
+    debug("_waitpid_target: job $job is complete now, reaping ...")
+	if $job->{debug} & 2;
+    $job->_mark_reaped;
+    return _reap_return($job);
+}
+
+sub _block_until_job_completes {
+    my ($job, $expire) = @_;
+    while ($job->{state} ne 'COMPLETE' && $job->{state} ne 'REAPED') {
+	return TIMEOUT if Time::HiRes::time() >= $expire;
+	__run_productive_waitpid_code();
+	Forks::Super::Util::pause();
+	if ($job->{state} =~ /DEFER|SUSPEND/) {
+	    Forks::Super::Queue::check_queue();
+	}
+    }
+    return 0;
 }
 
 sub _waitpid_name {
     my ($no_hang, $reap_bg_ok, $target, $timeout) = @_;
-    my $expire = Time::HiRes::time() + ($timeout || 9E9);
+    my $expire = Time::HiRes::time() + ($timeout || &FOREVER);
     my @jobs = Forks::Super::Job::getByName($target);
     if (@jobs == 0) {
 	return _bogus_waitpid_result();
@@ -396,10 +392,11 @@ sub _waitpid_name {
 	    # XXX - update $? ?
 	    return TIMEOUT;
 	}
-	$productive_waitpid_code->() if $productive_waitpid_code;
-	pause();
-	Forks::Super::Queue::run_queue()
-	    if grep {$_->{state} eq 'DEFERRED'} @jobs_to_wait_for;
+	__run_productive_waitpid_code();
+	Forks::Super::Util::pause();
+	if (grep {$_->{state} eq 'DEFERRED'} @jobs_to_wait_for) {
+	    Forks::Super::Queue::run_queue();
+	}
 	@jobs = grep { $_->{state} eq 'COMPLETE' 
 			   || $_->{state} eq 'REAPED'} @jobs_to_wait_for;
     }
@@ -410,22 +407,39 @@ sub _waitpid_name {
 # wait on any process from a specific process group
 sub _waitpid_pgrp {
     my ($no_hang, $reap_bg_ok, $target, $timeout) = @_;
-    my $expire = Time::HiRes::time() + ($timeout || 9E9);
+
+    if ($target == 0) {
+	if (! eval { $target = getpgrp(0) } ) {
+	    $target = $$;
+	}
+    } elsif ($target < 0) {
+	$target = -$target;
+    }
+
+    my $expire = Time::HiRes::time() + ($timeout || &FOREVER);
     my ($pid, $nactive) = _reap($reap_bg_ok,$target);
-    unless ($no_hang) {
+    if (! $no_hang) {
 	while (!isValidPid($pid,1) && $nactive > 0) {
 	    if (Time::HiRes::time() >= $expire) {
 		# XXX - update $? ?
 		return TIMEOUT;
 	    }
-	    $productive_waitpid_code->() if $productive_waitpid_code;
-	    pause();
+	    __run_productive_waitpid_code();
+	    Forks::Super::Util::pause();
 	    ($pid, $nactive) = _reap($reap_bg_ok,$target);
 	}
     }
-    $? = $ALL_JOBS{$pid}->{status}
-        if defined $ALL_JOBS{$pid};
+    if (defined $ALL_JOBS{$pid}) {
+	$? = $ALL_JOBS{$pid}{status};
+    }
     return $pid;
 }
 
+sub __run_productive_waitpid_code {
+    if ($productive_waitpid_code) {
+	$productive_waitpid_code->();
+    }
+    return;
+}
+
 1;
